from termcolor import cprint
from discord_webhook import DiscordEmbed, DiscordWebhook

class Network():
    MAINNET : str = "wss://stream.openseabeta.com/socket/websocket"
    TESTNET : str = "wss://testnets-stream.openseabeta.com/socket/websocket"

class EventTypes():
    ITEM_METADATA_UPDATED: str = 'item_metadata_updated'
    ITEM_LISTED : str = 'item_listed'
    ITEM_SOLD : str = 'item_sold'
    ITEM_TRANSFERRED : str = 'item_transferred'
    ITEM_RECEIVED_OFFER : str = 'item_received_offer'
    ITEM_RECEIVED_BID : str = 'item_received_bid'
    ITEM_CANCELLED : str = 'item_cancelled'

class Logger():
    def __init__(self, message: str) -> None:
        self.message = message

    def debug(self):
        cprint(f'[DEBUG]: {self.message}', 'cyan')

    def info(self):
        cprint(f'[INFO]: {self.message}', 'green')

    def warn(self):
        cprint(f'[WARN]: {self.message}', 'yellow')

    def error(self):
        cprint(f'[ERROR]: {self.message}', 'red')

class Webhook():

    OPENSEA_LOGO_URL = 'https://pbs.twimg.com/profile_images/1544105652330631168/ZuvjfGkT_400x400.png'
    FOOTER = 'Made by jerem.#3609'

    def __init__(self, webhook: DiscordWebhook, color: str = '03a9fc', thumbnail: str = None, footer: str = FOOTER):
        self.webhook = webhook
        self.color = color
        self.thumbnail = thumbnail
        self.footer = footer

    def ItemMetadataUpdated(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Metadata Updated !', color=self.color)

            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=str(name))

            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=str(chain_name))

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=str(image_url))
            elif self.thumbnail:
                embed.set_thumbnail(url=str(self.thumbnail))
            else:
                embed.set_thumbnail(url=str(self.OPENSEA_LOGO_URL))
                
            links = ''
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
                
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def ItemCancelled(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Item Cancelled !', color=self.color)
            
            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if base_price := payload['payload'].get('base_price'):
                embed.add_embed_field(name='Price :', value=str(int(base_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if payload['payload'].get('transaction') and payload['payload']['transaction'].get('hash'):
                hash_ = payload['payload']['transaction']['hash']
                links += f'[Order](https://etherscan.io/tx/{hash_}) • '
            if maker := payload['payload']['maker'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
                
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def ItemListed(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Item Listed !', color=self.color)
            
            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if base_price := payload['payload'].get('base_price'):
                embed.add_embed_field(name='Price :', value=str(int(base_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if maker := payload['payload']['maker'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))

            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def ItemSold(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Item Sold !', color=self.color)
            
            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if sale_price := payload['payload'].get('sale_price'):
                embed.add_embed_field(name='Price :', value=str(int(sale_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if payload['payload'].get('transaction') and payload['payload']['transaction'].get('hash'):
                hash_ = payload['payload']['transaction']['hash']
                links += f'[Order](https://etherscan.io/tx/{hash_}) • '
            if maker := payload['payload']['maker'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if taker := payload['payload']['taker'].get('address'):
                links += f'[Taker](https://etherscan.io/address/{taker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
                
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()
    
    def ItemTransferred(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Item Transferred !', color=self.color)
            
            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if sale_price := payload['payload'].get('sale_price'):
                embed.add_embed_field(name='Price :', value=str(int(sale_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)
            
            if quantity := payload['payload'].get('quantity'):
                embed.add_embed_field(name='Quantity :', value=str(quantity))

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if payload['payload'].get('transaction') and payload['payload']['transaction'].get('hash'):
                hash_ = payload['payload']['transaction']['hash']
                links += f'[Order](https://etherscan.io/tx/{hash_}) • '
            if maker := payload['payload']['from_account'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if taker := payload['payload']['to_account'].get('address'):
                links += f'[Taker](https://etherscan.io/address/{taker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
                
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def ItemReceivedOffer(self, payload: dict):
        print(payload)
        try:   
            embed = DiscordEmbed(title='New Item Received Offer !', color=self.color)

            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if base_price := payload['payload'].get('base_price'):
                embed.add_embed_field(name='Price :', value=str(int(base_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)

            if quantity := payload['payload'].get('quantity'):
                embed.add_embed_field(name='Quantity :', value=str(quantity))

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if maker := payload['payload']['maker'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if payload['payload'].get('taker') and payload['payload']['taker']['address']:
                taker = payload['payload']['taker']['address']
                links += f'[Taker](https://etherscan.io/address/{taker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
                
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def ItemReceivedBid(self, payload: dict):
        try:   
            embed = DiscordEmbed(title='New Item Receive Bid !', color=self.color)

            if name := payload['payload']['item']['metadata'].get('name'):
                embed.add_embed_field(name='Name :', value=name)

            if base_price := payload['payload'].get('base_price'):
                embed.add_embed_field(name='Price :', value=str(int(base_price)/10**18))
            
            if chain_name := payload['payload']['item']['chain'].get('name'):
                embed.add_embed_field(name='Chain :', value=chain_name)

            if quantity := payload['payload'].get('quantity'):
                embed.add_embed_field(name='Quantity :', value=str(quantity))

            if image_url := payload['payload']['item']['metadata'].get('image_url'):
                embed.set_thumbnail(url=image_url)
            elif self.thumbnail:
                embed.set_thumbnail(url=self.thumbnail)
            else:
                embed.set_thumbnail(url=self.OPENSEA_LOGO_URL)
            
            links = ''
            if maker := payload['payload']['maker'].get('address'):
                links += f'[Maker](https://etherscan.io/address/{maker}) • '
            if payload['payload'].get('taker') and payload['payload']['taker']['address']:
                taker = payload['payload']['taker']['address']
                links += f'[Taker](https://etherscan.io/address/{taker}) • '
            if metadata_url := payload['payload']['item']['metadata'].get('metadata_url'):
                links += f'[Metadata]({metadata_url}) • '
            if permalink := payload['payload']['item'].get('permalink'):
                links += f'[Asset]({permalink})'

            if links != '':
                embed.add_embed_field(name='Links :', value=str(links), inline=False)
            if self.footer:
                embed.set_footer(text=str(self.footer))
            return embed
        except Exception as e:
            Logger(f'Failed to build embed {e}').error()

    def send(self, payload: dict):
        try:
            if payload.get('event_type') == EventTypes.ITEM_METADATA_UPDATED:
                embed = self.ItemMetadataUpdated(payload)
            elif payload.get('event_type') == EventTypes.ITEM_CANCELLED:
                embed = self.ItemCancelled(payload)
            elif payload.get('event_type') == EventTypes.ITEM_LISTED:
                embed = self.ItemListed(payload)
            elif payload.get('event_type') == EventTypes.ITEM_SOLD:
                embed = self.ItemSold(payload)
            elif payload.get('event_type') == EventTypes.ITEM_TRANSFERRED:
                embed = self.ItemTransferred(payload)
            elif payload.get('event_type') == EventTypes.ITEM_RECEIVED_OFFER:
                embed = self.ItemReceivedOffer(payload)
            elif payload.get('event_type') == EventTypes.ITEM_RECEIVED_BID:
                embed = self.ItemReceivedBid(payload)
            else:
                Logger('Failed to get event type: ' + str(payload.get('event_type'))).error()
                return 

            self.webhook.remove_embeds()
            self.webhook.add_embed(embed=embed)
            self.webhook.execute()
        except Exception as e:
            Logger(f'Failled to execute webhook : {e}').error()



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = ['pip>=21.1', 'termcolor>=1.1.0', 'discord-webhook>=0.16.3', 'realtime>=0.0.4']

test_requirements = ['pytest>=3', ]

setup(
    author="Jerem",
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Office/Business :: Financial',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
        
    ],
    description="A python SDK for receiving updates from the OpenSea Stream API - pushed over websockets",
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords=['opensea', 'nft', 'non fungible token', 'crypto', 'stream'],
    maintainer='Jerem',
    name='opensea-stream',
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    test_suite='tests',
    url='https://github.com/jerempjerem/opensea-stream-py',
    project_urls={
        'Source': 'https://github.com/jerempjerem/opensea-stream-py'
    },
    version='0.1.0',
    zip_safe=False,
)




from realtime.connection import Socket
from realtime.channel import Channel
from .utils import *

class OpenseaStreamClient():
    def __init__(self, api_key: str, network: Network = Network.MAINNET):
        socketUrl = f'{network}?token={api_key}'
        self.socket = Socket(socketUrl)

        self.connected = False
        
    def connect(self):
        try:
            Logger('Connecting to socket').debug()
            self.socket.connect()
            self.connected = True
        except Exception as e:
            Logger(f'Failed to connect to socket : {e}').error()
    
    def createChannel(self, collection_slug: str):
        try:
            channel : Channel = self.socket.set_channel(f'collection:{collection_slug}')
            channel.join()
            Logger(f'Successfully joined channel {collection_slug}').info()
            return channel
        except Exception as e:
            Logger(f'Failed to join channel : {e}').error()

    def getChannels(self, collection_slug: str):
        try:
            if collection_slug not in list(self.socket.channels):
                Logger(f'Creating channel for topic: {collection_slug}').debug()
                return self.createChannel(collection_slug)
            Logger(f'Channel for topic: {collection_slug} already exist').warn()
        except Exception as e:
            Logger(f'Failed to create channel for topic {collection_slug} : {e}').error()

    def on(self, event_type: EventTypes, collection_slug: str, callback):
        try:
            if not self.connected:
                self.socket.connect()
                self.connected = True
            channel = self.getChannels(collection_slug)
            Logger(f'Subscribing to {event_type} events on {collection_slug}').info()
            channel.on(event_type, callback)
        except Exception as e:
            Logger(e).error()

    def startListening(self):
        try:
            Logger(f'Channel joined : {list(self.socket.channels)}').info()
            Logger('Listenning').warn()
            self.socket.listen()
        except Exception as e:
            Logger(e).error()

    def onItemMetadataUpdated(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item metadata updates on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_METADATA_UPDATED, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onItemCancelled(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item cancellations on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_CANCELLED, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onItemListed(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item listings on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_LISTED, collection_slug, callback)
        except Exception as e:
            Logger(e).error()
    
    def onItemSold(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item sales on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_SOLD, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onItemTransferred(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item transfers on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_TRANSFERRED, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onItemReceivedOffer(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item offers on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_RECEIVED_OFFER, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onItemReceivedBid(self, collection_slug: str, callback):
        try:
            Logger(f"Listening for item offers on {collection_slug}").debug()
            return self.on(EventTypes.ITEM_RECEIVED_BID, collection_slug, callback)
        except Exception as e:
            Logger(e).error()

    def onEvents(self, slugs: list, event_type: EventTypes, callback):
        try:
            if type(slugs) != list:
                Logger('slugs params must be a list').error()
                return
            if type(event_type) != list:
                Logger('slugs params must be a list').error()
                return
            for slug in slugs:
                for event in event_type:
                    self.on(event, slug, callback)
        except Exception as e:
            Logger(e).error()